package com.lilosoft.complex.matter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.lilosoft.complex.AbstractComplexService;
import com.lilosoft.complex.ComplexApiRest;
import com.lilosoft.complex.Rest;
import com.lilosoft.complex.matter.model.req.AcceptReq;
import com.lilosoft.complex.matter.model.req.ImplementationReq;
import com.lilosoft.complex.matter.model.req.DeviceLoginReq;
import com.lilosoft.complex.matter.model.rsp.AcceptRspInfo;
import com.lilosoft.complex.matter.model.rsp.LoginRspInfo;
import com.lilosoft.complex.matter.model.rsp.MatterListInfo;
import com.lilosoft.complex.matter.model.rsp.MatterRespInfo;
import com.lilosoft.complex.matter.model.rsp.matter.MatterWorkInfo;
import com.lilosoft.complex.matter.model.rsp.work.WorkInfo;
import com.lilosoft.complex.matter.service.IComplexMatterService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 综合窗口业务实现类
 *
 * @author: zxfei
 * @date: 2023/10/30 14:53
 */
@Service("complexMatterService")
@Slf4j
public class ComplexMatterServiceImpl extends AbstractComplexService implements IComplexMatterService {

    //token有效期 10小时
    @Override
    public Rest<LoginRspInfo> deviceLogin(DeviceLoginReq loginReq) {
        String deviceLoginApi = host + "self-device-info/login";
        String rest = HttpUtil.post(deviceLoginApi, JSON.toJSONString(loginReq));
        ComplexApiRest<LoginRspInfo> complexApiRest = JSON.parseObject(rest, new TypeReference<ComplexApiRest<LoginRspInfo>>() {
        });
        if (complexApiRest.getCode() == 200) {
            //更新token
            String deviceLoginToken = complexApiRest.getResult().getToken();
            String deviceLogo = complexApiRest.getResult().getSelfDeviceInfo().getDeviceLogo();
            this.tokenMap.put(deviceLogo, deviceLoginToken);
            return Rest.ok(complexApiRest.getMessage(), complexApiRest.getResult());
        }
        return Rest.fail(complexApiRest.getMessage());
    }

    @Override
    public Rest<String> uploadFile(String deviceLogo, byte[] bytes, String fileName) {

        String uploadApi = host + "common/uploadFileToFtp?conversionImage=Y";
        HttpRequest request = HttpRequest.post(uploadApi);

        String deviceToken = checkToken(deviceLogo);

        request.form("file", bytes, fileName);
        String uploadRest = request.header("X-Access-Token", deviceToken).execute().body();
        ComplexApiRest<String> complexApiRest = JSON.parseObject(uploadRest, new TypeReference<ComplexApiRest<String>>() {
        });
        if (complexApiRest.getCode() == 200) {
            return Rest.ok(complexApiRest.getMessage(), complexApiRest.getResult());
        }
        return Rest.fail(complexApiRest.getMessage());
    }

    @Override
    public Rest<AcceptRspInfo> accept(String deviceLogo, AcceptReq acceptReq) {
        String deviceLoginApi = host + "self-device-info/acceptHandling";

        log.info("accept req==>{}",JSON.toJSONString(acceptReq));

        String deviceToken = checkToken(deviceLogo);
        String rest = HttpUtil.createPost(deviceLoginApi)
                .header("X-Access-Token", deviceToken)
                .body(JSON.toJSONString(acceptReq))
                .execute().body();
        ComplexApiRest<AcceptRspInfo> complexApiRest = JSON.parseObject(rest, new TypeReference<ComplexApiRest<AcceptRspInfo>>() {
        });
        if (complexApiRest.getCode() == 200) {
            return Rest.ok(complexApiRest.getMessage(), complexApiRest.getResult());
        }
        return Rest.fail(complexApiRest.getMessage());
    }


    @Override
    public Rest<MatterListInfo> findMatters(String deviceLogo, ImplementationReq implementationReq) {
        String mattersApi = host + "event-implementation/list";

        Map<String, String> paramMap = new HashMap<>();
        // 将Map中的属性值转为字符串
        for (Map.Entry<String, Object> entry : BeanUtil.beanToMap(implementationReq, false, true).entrySet()) {
            String key = entry.getKey();
            Object value = entry.getValue();
            String stringValue = String.valueOf(value);
            paramMap.put(key, stringValue);
        }

        String deviceToken = checkToken(deviceLogo);

        String rest = HttpUtil.createGet(mattersApi)
                .header("X-Access-Token", deviceToken)
                .formStr(paramMap)
                .execute().body();

        ComplexApiRest<MatterListInfo> complexApiRest = JSON.parseObject(rest, new TypeReference<ComplexApiRest<MatterListInfo>>() {
        });
        if (complexApiRest.getCode() == 200) {
            return Rest.ok(complexApiRest.getMessage(), complexApiRest.getResult());
        }
        return Rest.fail(complexApiRest.getMessage());
    }

    @Override
    public Rest<MatterWorkInfo> doWorkMatterSearch(String deviceLogo, String eventIds) {
        String mattersApi = host + "bus-situation-material/getSituationMaterialTreeByEventIds";

        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("eventIds", eventIds);

        String deviceToken = checkToken(deviceLogo);
        String rest = HttpUtil.createGet(mattersApi)
                .header("X-Access-Token", deviceToken)
                .formStr(paramMap)
                .execute().body();

        ComplexApiRest<List<MatterWorkInfo>> complexApiRest = JSON.parseObject(rest, new TypeReference<ComplexApiRest<List<MatterWorkInfo>>>() {
        });
        if (complexApiRest.getCode() == 200) {
            if(!ObjectUtils.isEmpty(complexApiRest.getResult())){
                return Rest.ok(complexApiRest.getMessage(), complexApiRest.getResult().get(0));
            }else {
                return Rest.fail("查询结果数量为0");
            }
        }
        return Rest.fail(complexApiRest.getMessage());
    }

    @Override
    public Rest<WorkInfo> getWorkMatter(String deviceLogo, String eventIds) {
        String mattersApi = host + "event-implementation/getDynamicFormByEventIds";

        ArrayList<String> list = new ArrayList<>();
        list.add(eventIds);

        String deviceToken = checkToken(deviceLogo);

        String rest = HttpUtil.createPost(mattersApi)
                .header("X-Access-Token", deviceToken)
                .body(JSON.toJSONString(list))
                .execute().body();

        ComplexApiRest<WorkInfo> complexApiRest = JSON.parseObject(rest, new TypeReference<ComplexApiRest<WorkInfo>>() {
        });
        if (complexApiRest.getCode() == 200) {
            return Rest.ok(complexApiRest.getMessage(), complexApiRest.getResult());
        }
        return Rest.fail(complexApiRest.getMessage());
    }


    private String checkToken(String deviceLogo) {
        String deviceToken = tokenMap.getOrDefault(deviceLogo, "");
        if (ObjectUtils.isEmpty(deviceToken)) {
            deviceToken = getToken(deviceLogo);
        }
        return deviceToken;
    }

    public static void main(String[] args) {
        // Define the URL where you want to upload the file
        String uploadUrl = "http://example.com/upload";

        // Define the path of the file you want to upload
        String filePath = "/path/to/your/file.txt";

        byte[] bytes = "111".getBytes();

        // Create an HTTP request
        HttpRequest request = HttpRequest.post(uploadUrl);

        // Attach the file to the request
        request.form("file", bytes, "1.png");

        // Send the request and get the response
        HttpResponse response = request.execute();

        String body = request.execute().body();

        // Check the response status
        int status = response.getStatus();
        if (status == 200) {
            System.out.println("File uploaded successfully!");
        } else {
            System.out.println("File upload failed. Status code: " + status);
        }

    }
}
